#include <nds.h>
#include <math.h>
#include <nds/arm9/sound.h>
#include <nds/interrupts.h>
#include "mathematics.h"
#include "camera.h"
#include "metacube.h"
#include "gfx_engine.h"
#include "intro.h"
#include "gameover.h"
#include "time.h"
#include "global.h"
#include "persistent.h"
#include "mainloop.h"
#include "highscores.h"
#include "splash.h"
#include "stupid.h"
#include "cup.h"


#define FOV (35.0f)

int main ()
{
  s8 score_rank ;

  powerON (POWER_ALL) ;

  vramSetMainBanks (VRAM_A_LCD, VRAM_B_LCD, VRAM_C_LCD, VRAM_D_LCD) ;
  irqInit () ;
  irqEnable (IRQ_VBLANK) ;
  irqSet (IRQ_TIMER2, Timer2Handler) ;

  /* Forces the main core to display on the top. */
  lcdMainOnTop () ;
  setGenericSound (11025,  /* Sample rate. */
		   127,	   /* Volume. */
		   64,	   /* Panning. */
		   1) ;	   /* Sound format. */

  /* Performs first randomizer initialisation as a side effect. */
  SplashNeo () ;

  PlayStupidGeneric () ;

  ReadScores () ;
  SetActiveCubeBuffer0 () ;
  if (LoadHibernated ()) goto direct_run_loaded_hibernated ;
  for (;;) {
    NewGame (Intro ()) ;
  direct_run_loaded_hibernated:
    InitCamera (RAD (FOV), RAD (FOV), &global_camera) ;
    Init3DGfxEngine () ;
    Init2DMainGameScreen (true) ;
    GameMainLoop () ;
    Gameover (false) ;
    /* Check and save the highscores. */
    score_rank = CheckIfHighscore (global_game.score, global_game.nb_lines) ;
    if (score_rank >= 0) {
      ShowCup (score_rank) ;
      SaveScores () ;
    }
    /* Show the current game type highscores. */
    HighScores (global_game.game_type, score_rank) ;
  }

  return (0) ;
}
