#include <nds.h>
#include <math.h>
#include "mathematics.h"
#include "camera.h"


#define INITIAL_CAMERA_X (0.0f)
#define INITIAL_CAMERA_Y (4.0f)
#define INITIAL_CAMERA_Z (3.0f)
#define DISPLACEMENT_STEP (0.1f)


void InitCamera (float h_aperture, float v_aperture, struct camera *out)
{
  struct point around_right = { 1.0f, 0.0f, 0.0f } ;

  out->position.x = INITIAL_CAMERA_X ;
  out->position.y = INITIAL_CAMERA_Y ;
  out->position.z = INITIAL_CAMERA_Z ;

  /* Never change below. All below is hardwired-computed from position. */
  out->dir.x = 0.0f ;
  out->dir.y = 0.0f ;
  out->dir.z = -1.0f ;

  out->right.x = 1.0f ;
  out->right.y = 0.0f ;
  out->right.z = 0.0f ;

  out->up.x = 0.0f ;
  out->up.y = 1.0f ;
  out->up.z = 0.0f ;

  /* Initial rotation. */
  RotateCameraAroundArbitrary (out, &around_right, RAD (45.0f)) ;
}





void RenderCamera (struct camera *camera)
{
  /* The point at which the camera looks. */
  struct point viewpoint ;

  VectorsAdd (&camera->position, &camera->dir, &viewpoint) ;
  gluLookAt (camera->position.x, camera->position.y, camera->position.z,
	     viewpoint.x, viewpoint.y, viewpoint.z,
	     camera->up.x, camera->up.y, camera->up.z) ;
}




/* Rotate vector v around arbitrary axis for angle radians. It can only */
/* rotate around an axis through our object. To rotate around another   */
/* axis:                                                                */
/*    - first translate the object to the axis                          */
/*    - then use this function,                                         */
/*    - then translate back in the new direction.                       */
void RotateCameraAroundArbitrary (struct camera *camera, struct point *axis,
				  float angle)
{
  /* Needs optimisation to share computation ! TODO ! */
  RotateAroundArbitrary (&camera->right, axis, angle, &camera->right) ;
  RotateAroundArbitrary (&camera->up, axis, angle, &camera->up) ;
  RotateAroundArbitrary (&camera->dir, axis, angle, &camera->dir) ;
}



void MoveFoward (struct camera *camera)
{
  struct point tmp ;
  VectorScale (&camera->dir, DISPLACEMENT_STEP, &tmp) ;
  VectorsAdd (&camera->position, &tmp, &camera->position) ;
}



void MoveBackward (struct camera *camera)
{
  struct point tmp ;
  VectorScale (&camera->dir, -DISPLACEMENT_STEP, &tmp) ;
  VectorsAdd (&camera->position, &tmp, &camera->position) ;
}


void ZoomReset (struct camera *camera)
{
  camera->position.x = INITIAL_CAMERA_X ;
  camera->position.y = INITIAL_CAMERA_Y ;
  camera->position.z = INITIAL_CAMERA_Z ;
}

