#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "gesture.h"



static unsigned short StrokeSize (struct stroke *stroke)
{
  unsigned short len = 0 ;
  while (stroke != NULL) {
    len++ ;
    stroke = stroke->next ;
  }
  return (len) ;
}




struct stroke* ConsStroke (struct stroke *list, struct gesture_point *point)
{
  struct stroke *tmp ;

  tmp = (struct stroke *) malloc (sizeof (struct stroke)) ;
  if (tmp == NULL) return (NULL) ;
  tmp->point.x = point->x ;
  tmp->point.y = point->y ;
  tmp->next = list ;
  return (tmp) ;
}



static void NormalizeStrokeSizeInPlace (struct stroke *stroke)
{
  float minX = MAXFLOAT ;
  float maxX = -MAXFLOAT ;
  float minY = MAXFLOAT ;
  float maxY = -MAXFLOAT ;
  float width, height, scale ;
  struct stroke *j ;

  for (j = stroke; j != NULL; j = j->next) {
    struct gesture_point pt ;
    pt.x = j->point.x ;
    pt.y = j->point.y ;

    if (minX > pt.x) minX = pt.x ;
    if (minY > pt.y) minY = pt.y ;
    if (maxX < pt.x) maxX = pt.x ;
    if (maxY < pt.y) maxY = pt.y ;
  }

  width = maxX - minX ;
  height = maxY - minY ;

  scale = (width > height) ? width : height ;
  if (scale <= 0.0f) return ;
  scale = 1.0f / scale ;

  for (j = stroke; j != NULL; j = j->next) {
    j->point.x *= scale ;
    j->point.y *= scale ;
  }
}





static float GetStrokeLength (struct stroke *stroke)
{
  float len = 0.0f ;
  struct stroke *i = stroke ;
  struct gesture_point startPt, endPt ;
  float dx, dy ;

  if (StrokeSize (stroke) <= 1) return (0.0f) ;

  startPt.x = i->point.x ;
  startPt.y = i->point.y ;

  i = i->next ;
  while (i != NULL) {
    endPt.x = i->point.x ;
    endPt.y = i->point.y ;
    dx = endPt.x - startPt.x ;
    dy = endPt.y - startPt.y ;

    len += sqrtf (dx * dx + dy * dy) ;
    startPt.x = endPt.x ;
    startPt.y = endPt.y ;
    i = i->next ;
  }
  return (len) ;
}




#define KPOINTSPERSTROKE 32

static struct stroke* CopyAndNormalizeStrokeSpacing (struct stroke *oldStroke)
{
  /* Distance along old stroke at the end of the current segment. */
  float endOldDist = 0.0f ;
  /* Distance along the old stroke at the beginning of the current segment. */
  float startOldDist = 0.0f ;
  /* Distance along new stroke. */
  float newDist = 0.0f ;
  /* Length of the current segment (on the old stroke). */
  float currSegmentLen = 0.0f ;
  struct stroke *newStroke = NULL ;
  struct stroke *i ;
  struct gesture_point startPt, endPt ;
  float newSegmentLen ;

  newSegmentLen = GetStrokeLength (oldStroke) ;

  if ((StrokeSize (oldStroke) <= 1) || (newSegmentLen <= 0.0f))
    return (NULL) ;

  newSegmentLen /= (KPOINTSPERSTROKE - 1) ;
  i = oldStroke ;

  /* Add the first point to the new stroke. */
  newStroke = ConsStroke (newStroke, &i->point) ;

  startPt.x = i->point.x ;
  startPt.y = i->point.y ;
  endPt.x = i->point.x ;
  endPt.y = i->point.y ;
  i = i->next ;

  for (;;) {
    float excess = endOldDist - newDist ;
    /* We have accumulated enough length, add a point. */
    if (excess >= newSegmentLen) {
      float ratio ;
      struct gesture_point newPt ;

      newDist += newSegmentLen ;
      ratio = (newDist - startOldDist) / currSegmentLen ;

      newPt.x = (endPt.x - startPt.x) * ratio + startPt.x ;
      newPt.y = (endPt.y - startPt.y) * ratio + startPt.y ;
      newStroke = ConsStroke (newStroke, &newPt) ;
    }
    else {
      float dx ;
      float dy ;
      
      if (i == NULL) break ;

      /* Store the start of the current segment. */
      startPt.x = endPt.x ;
      startPt.y = endPt.y ;
      endPt.x = i->point.x ;
      endPt.y = i->point.y ;
      i = i->next ;
      dx = endPt.x - startPt.x ;
      dy = endPt.y - startPt.y ;

      /* Start accumulated distance (along the old stroke) */
      /* at the beginning of the segment.                  */
      startOldDist = endOldDist ;
      /* Add the length of the current segment */
      /* to the total accumulated length.      */
      currSegmentLen = sqrtf (dx * dx + dy * dy) ;
      endOldDist += currSegmentLen ;
    }
  }

  if (StrokeSize(newStroke) < KPOINTSPERSTROKE) {
    newStroke = ConsStroke (newStroke, &endPt) ;
  }

  return (newStroke) ;
}




static void NormalizeStrokeCenterInPlace (struct stroke *stroke)
{
  float centerX = 0.0f ;
  float centerY = 0.0f ;
  int pointCount ;
  struct stroke *j ;

  /* Calcul du barycentre du graffiti. */
  pointCount = StrokeSize (stroke) ;
  for (j = stroke; j != NULL; j = j->next) {
    centerX += j->point.x ;
    centerY += j->point.y ;
  }

  if (pointCount <= 0 ) return ;

  centerX /= pointCount ;
  centerY /= pointCount ;

  /* On dplace le graffiti  l'origine et on soustrait les */
  /* coordonnes du barycentre  chaque point. */
  //from point coordinates
  for (j = stroke; j != NULL; j=j->next) {
    j->point.x -= centerX ;
    j->point.y -= centerY ;
  }
}






static float StrokeDotProduct (struct stroke *stroke1,
			       struct stroke *stroke2)
{
  struct stroke *j1, *j2 ;
  struct gesture_point pt1, pt2 ;
  float dotProduct = 0.0f ;

  if (StrokeSize (stroke1) != StrokeSize (stroke2)) return (-1.0f) ;
  for (j1 = stroke1, j2 = stroke2; (j1 != NULL) && (j2 != NULL);
       j1 = j1->next, j2 =j2->next) {
    pt1.x = j1->point.x ;
    pt1.y = j1->point.y ;
    pt2.x = j2->point.x ;
    pt2.y = j2->point.y ;
    dotProduct += pt1.x * pt2.x + pt1.y * pt2.y ;
  }
  return (dotProduct) ;
}




void FreeStroke (struct stroke *stroke)
{
  struct stroke *tmp_s, *tmp_s_next ;

  tmp_s = stroke ;
  while (tmp_s != NULL) {
    tmp_s_next = tmp_s->next ;
    free (tmp_s) ;
    tmp_s = tmp_s_next ;
  }
}




/* *********************************** */
/* Fonctions exportes  partir d'ici. */
/* *********************************** */

/* Attention, c'est destructeur sur la stroke passe en argument: */
/* Elle se retrouve mise  l'chelle.                             */
/* Par contre ne change pas la structure mmoire.                 */
/* Warning, destructive on the arument stroke: scales it.  */
/* However, does not change the memory allocation pattern. */
struct stroke* BuildFullNormalisedStroke (struct stroke* stroke)
{
  struct stroke *tmp_s ;

  if (stroke == NULL) return (NULL) ;
  NormalizeStrokeSizeInPlace (stroke) ;
  tmp_s = CopyAndNormalizeStrokeSpacing (stroke) ;
  NormalizeStrokeCenterInPlace (tmp_s) ;
  return (tmp_s) ;
}




float StrokeMatchP (struct stroke *stroke1, struct stroke *stroke2)
{
  float score ;

  if ((stroke1 == NULL) || (stroke2 == NULL)) return (0.0f) ;

  score = StrokeDotProduct (stroke1, stroke2) ;
  if (score <= 0.0f) return (0.0f) ;

  /* Rappel sur la formule sur le cosinus de l'angle entre 2 vecteurs. */
  /* cos a =                                                           */
  /*     (u.v) / (sqrt (u.u) * sqrt (v.v)) =                           */
  /*     (u.v) / sqrt ((u.u) * (v.v))                                  */
  score /= sqrtf (StrokeDotProduct (stroke1, stroke1) *
		  StrokeDotProduct (stroke2, stroke2)) ;
  return (score) ;
}
