#include <nds.h>
#include <nds/arm9/sound.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "mathematics.h"
#include "camera.h"
#include "metacube.h"
#include "homebrew_cube.h"
#include "gfx_engine.h"
#include "printing.h"
#include "global.h"
#include "time.h"
#include "gfx_load.h"
#include "time.h"
#include "info_game_gfx.h"
#include "info_game_fonts_gfx.h"
#include "sprites_gfx.h"
#include "sleep_anim_gfx.h"
#include "deep_space.h"
/* Textures: raw 32x32 & 128x128 16 bits images (converted to 8bpp as PCX). */
#include "metacube_cube_texture_bin.h"
#include "flipped_cube_texture_bin.h"
#include "metacube_cube_texture_shaded_bin.h"
#include "flipped_cube_texture_shaded_bin.h"
#include "deep_space_texture_bin.h"
#include "wav_bubble_pop.h"
#include "wav_rotate.h"
#include "wav_dont_fit.h"
#include "wav_disapear.h"
#include "wav_scoring.h"


int texture_ids[NB_TEXTURES] = { 0, 0, 0, 0, 0 } ;





struct sSpriteEntry global_sprites_copy[128] ;





void RefreshScoreAndLines (u32 old_score, u32 old_nb_lines)
{
  u8 i ;
  u32 div, rem, counter, factor ;

  StopGameTimer () ;

  /* Score refresh. */
  div = global_game.score - old_score ;
  if (div == 0) {
    LargePrintInt (((u16*) SCREEN_BASE_BLOCK_SUB (0)), global_game.score,
		   SCORE_X_OFFSET, SCORE_Y_OFFSET,
		   EMPTY_INFO_GAME_FONT_TILE_INDEX,
		   ZERO_INFO_GAME_FONT_TILE_INDEX,
		   INFO_GAME_FONT_SIZE_IN_TILES) ;
  }            /* End of if (div == 0). */
  else {
    factor = 1 ;
    counter = old_score ;
    while (div != 0) {
      rem = div % 10 ;
      for (i = 0; i < rem; i++) {
	counter = counter + factor ;
	playGenericSound (g_wav_scoring, WAV_SCORING_LENGTH * sizeof (u32)) ;
	LargePrintInt (((u16*) SCREEN_BASE_BLOCK_SUB (0)), counter,
		       SCORE_X_OFFSET, SCORE_Y_OFFSET,
		       EMPTY_INFO_GAME_FONT_TILE_INDEX,
		       ZERO_INFO_GAME_FONT_TILE_INDEX,
		       INFO_GAME_FONT_SIZE_IN_TILES) ;
	swiWaitForVBlank () ;
	swiDelay (500000) ;
      }         /* End of for (i = 0; i < rem; i++). */
      div = div / 10 ;
      factor *= 10 ;
    }           /* End of while (div != 0). */
  }             /* End of else. */

  /* Number of lines refresh. */
  div = global_game.nb_lines - old_nb_lines ;
  if (div == 0) {
    LargePrintInt (((u16*) SCREEN_BASE_BLOCK_SUB (0)), global_game.nb_lines,
		   LINES_X_OFFSET, LINES_Y_OFFSET,
		   EMPTY_INFO_GAME_FONT_TILE_INDEX,
		   ZERO_INFO_GAME_FONT_TILE_INDEX,
		   INFO_GAME_FONT_SIZE_IN_TILES) ;
  }            /* End of if (div == 0). */
  else {
    factor = 1 ;
    counter = old_nb_lines ;
    while (div != 0) {
      rem = div % 10 ;
      for (i = 0; i < rem; i++) {
	counter = counter + factor ;
	playGenericSound (g_wav_scoring, WAV_SCORING_LENGTH * sizeof (u32)) ;
	LargePrintInt (((u16*) SCREEN_BASE_BLOCK_SUB (0)), counter,
		       LINES_X_OFFSET, LINES_Y_OFFSET,
		       EMPTY_INFO_GAME_FONT_TILE_INDEX,
		       ZERO_INFO_GAME_FONT_TILE_INDEX,
		       INFO_GAME_FONT_SIZE_IN_TILES) ;
	swiWaitForVBlank () ;
	swiDelay (200000) ;
      }         /* End of for (i = 0; i < rem; i++). */
      div = div / 10 ;
      factor *= 10 ;
    }           /* End of while (div != 0). */
  }             /* End of else. */

  StartGameTimer () ;
}





void Init3DGfxEngine ()
{
  videoSetMode (0) ;
  vramSetBankA (VRAM_A_TEXTURE) ;
  glViewPort (0, 0, 255, 191) ;
  glClearColor (0, 0,0) ;
  glClearDepth (0x7FFF) ;

  glResetTextures () ;
  /* Allocate room for the textures. */
  glGenTextures (NB_TEXTURES, texture_ids) ;

  /* Texture for the metacube sub-cubes. */
  glBindTexture (GL_TEXTURE_2D, texture_ids[METACUBE_CUBE_TEXTURE_ID]) ;
  glTexImage2D (GL_TEXTURE_2D, 0, GL_RGB,
		TEXTURE_SIZE_64, TEXTURE_SIZE_64, 0,
		TEXGEN_TEXCOORD, (u8*) &metacube_cube_texture_bin) ;

  /* Texture for the outter limit of the metacube. */
  glBindTexture (GL_TEXTURE_2D, texture_ids[FLIPPED_CUBE_TEXTURE_ID]) ;
  glTexImage2D (GL_TEXTURE_2D, 0, GL_RGB,
		TEXTURE_SIZE_128, TEXTURE_SIZE_128, 0,
		TEXGEN_TEXCOORD, (u8*) &flipped_cube_texture_bin) ;

  /* Texture for the metacube sub-cubes when shaded. */
  glBindTexture (GL_TEXTURE_2D, texture_ids[METACUBE_CUBE_TEXTURE_SHADED_ID]) ;
  glTexImage2D (GL_TEXTURE_2D, 0, GL_RGB,
		TEXTURE_SIZE_64, TEXTURE_SIZE_64, 0,
		TEXGEN_TEXCOORD, (u8*) &metacube_cube_texture_shaded_bin) ;

  /* Texture for the outter limit of the metacube when shaded. */
  glBindTexture (GL_TEXTURE_2D, texture_ids[FLIPPED_CUBE_TEXTURE_SHADED_ID]) ;
  glTexImage2D (GL_TEXTURE_2D, 0, GL_RGB,
		TEXTURE_SIZE_64, TEXTURE_SIZE_64, 0,
		TEXGEN_TEXCOORD, (u8*) &flipped_cube_texture_shaded_bin) ;

  /* Texture for the deep space. */
  glBindTexture (GL_TEXTURE_2D, texture_ids[DEEP_SPACE_TEXTURE_ID]) ;
  glTexImage2D (GL_TEXTURE_2D, 0, GL_RGB,
		TEXTURE_SIZE_128, TEXTURE_SIZE_128, 0,
		TEXGEN_TEXCOORD, (u8*) &deep_space_texture_bin) ;

  videoSetMode (MODE_0_3D) ;
}





void UpdateTimer ()
{
  int hours, minutes, seconds ;
  u32 elapsed_seconds = 0 ;

  switch (global_game.game_type) {
  case Game_Type_Endless:
    elapsed_seconds = global_elapsed_semi_seconds / 2 ; 
    break ;
  case Game_Type_5mn:
    elapsed_seconds = (MINUTE5 - global_elapsed_semi_seconds) / 2 ; 
    break ;
  case Game_Type_10mn:
    elapsed_seconds = (MINUTE10 - global_elapsed_semi_seconds) / 2 ; 
    break ;
  case Game_Type_15mn:
    elapsed_seconds = (MINUTE15 - global_elapsed_semi_seconds) / 2 ; 
    break ;
  }
  hours = elapsed_seconds / (60 * 60) ;
  minutes = (elapsed_seconds - (hours * 60 * 60)) / 60 ;
  seconds = elapsed_seconds - (hours * 60 * 60) - (minutes * 60) ;
  Print2DigitsInt (((u16*) SCREEN_BASE_BLOCK_SUB (0)), hours,
		   TIMER_HOURS_X_OFFSET, TIMER_Y_OFFSET) ;
  Print2DigitsInt (((u16*) SCREEN_BASE_BLOCK_SUB (0)), minutes,
		   TIMER_MINUTES_X_OFFSET, TIMER_Y_OFFSET) ;
  Print2DigitsInt (((u16*) SCREEN_BASE_BLOCK_SUB (0)), seconds,
		   TIMER_SECONDS_X_OFFSET, TIMER_Y_OFFSET) ;
}



void HideAllSprites ()
{
  u16 i ;

  /* On dsactive les sprites. */
  for (i = 0; i < 128; i++)
    global_sprites_copy[i].attribute[0] = ATTR0_DISABLED ;
  swiWaitForVBlank () ;
  memcpy
    ((void*) OAM_SUB, (void*) global_sprites_copy,
     128 * sizeof (struct sSpriteEntry)) ;
}





void Init2DMainGameScreen (bool show)
{
  s16 i ;

  vramSetBankC (VRAM_C_SUB_BG_0x6200000);     /* Bg 0x06200000 - 0x06040000. */
  vramSetBankD (VRAM_D_SUB_SPRITE);           // Objs tiles 0x6600000

  SUB_BG0_CR =
       BG_MAP_BASE (0)        // Map (pas adresse).
     | BG_TILE_BASE (1)       // Tiles (pas adresse).
     | BG_PRIORITY_3          // Priorit la plus faible.
     | BG_COLOR_256           // 256 couleurs.
     | TEXTBG_SIZE_256x256 ;  // 256 x 256 pixels

  load_palette_tile_map_256
    ((u16*) CHAR_BASE_BLOCK_SUB (1),     // TILES (16ko aligned) (address)
     (u16*) SCREEN_BASE_BLOCK_SUB (0),   // MAP (2ko aligned) (address)
     INFO_GAME_PALETTE_SIZE,
     info_game_palette,
     NB_INFO_GAME_TILES,
     info_game_tiles,
     INFO_GAME_MAP_WIDTH,
     INFO_GAME_MAP_HEIGHT,
     info_game_map,
     Video_sub) ;

  load_palette_tile_map_256
    (((u16*) CHAR_BASE_BLOCK_SUB (1)) +
     INFO_GAME_FONTS_TILES_LOADING_OFFSET,   // TILES (16ko aligned) (address)
     NULL,                               // MAP (2ko aligned) (address)
     0,
     NULL,
     NB_INFO_GAME_FONTS_TILES,
     info_game_fonts_tiles,
     0,
     0,
     NULL,
     Video_sub) ;

  load_sprites_256
    (SPRITES_LOADING_OFFSET,
     NB_SPRITES_TILES,
     sprites_tiles,
     SPRITES_PALETTE_SIZE,
     sprites_palette,
     Video_sub) ;

  HideAllSprites () ;

  if (show) {
    /* Screen appears fading. */
    SUB_BLEND_Y = 15 ;
    SUB_BLEND_CR = BLEND_FADE_BLACK | BLEND_SRC_BG0 ;
    videoSetModeSub (MODE_0_2D | DISPLAY_BG0_ACTIVE |
		     DISPLAY_SPR_1D_LAYOUT | DISPLAY_SPR_ACTIVE) ;
    for (i = 31; i > 0; i--) {
      swiWaitForVBlank () ;
      SUB_BLEND_Y = (u16) (i / 2) ;
    }
  }
}





static void RenderShape ()
{
  u16 indice_x, indice_y, indice_z ;
  float world_x, world_y, world_z ;

  for (indice_y = 0; indice_y < current_shape.ysize; indice_y++) {
    for (indice_x = 0; indice_x < current_shape.xsize; indice_x++) {
      for (indice_z = 0; indice_z < current_shape.zsize; indice_z++) {
	if (current_shape.data[indice_x][indice_y][indice_z] != Empty_Cell) {
	  /* The shape's cell is occupied. */
	  world_x =
	    ((indice_x + current_shape.xorigin) -
	     (((float) (MCUBEDIM - 1)) / 2.0f)) *
	    CUBE_EDGE_SIZE ;
	  world_y =
	    ((indice_y + current_shape.yorigin) -
	     (((float) (MCUBEDIM - 1)) / 2.0f)) *
	    CUBE_EDGE_SIZE ;
	  world_z =
	    - ((indice_z + current_shape.zorigin) -
	       (((float) (MCUBEDIM - 1)) / 2.0f)) *
	    CUBE_EDGE_SIZE ;
	  RenderHomebrewCube ((CUBE_EDGE_SIZE / 2.0f),
			      world_x, world_y, world_z, false) ;
	} /* End of if (current_shape.data[indice_x][indice_y][indice_z] !=. */
      }     /* End of for (indice_z = 0; indice_z < current_shape.zsize;... */
    }       /* End of for (indice_x = 0; indice_x < current_shape.xsize;... */
  }         /* End of for (indice_y = 0; indice_y < current_shape.ysize;... */
}




#define HULL_SHADE_Y (-((MCUBEDIM / 2.0f) * CUBE_EDGE_SIZE) + 0.08f)
static void ShadedTopCubeFace (float radius, float cx, float cz)
{
  float x_neg = -radius + cx ;
  float x_pos = radius + cx ;
  float z_neg = -radius + cz ;
  float z_pos = radius + cz ;

  glBegin (GL_QUADS) ;
  /* Top Face: points up on y. */
  glNormal3f (0.0f, 1.0f, 0.0f) ;
  glTexCoord2f (0.0f, SCALE_TO_TEXTURE_64 (1.0f)) ;
  glVertex3f (x_neg, HULL_SHADE_Y, z_neg) ;  /* 5 */
  glTexCoord2f (0.0f, 0.0f) ;
  glVertex3f (x_neg, HULL_SHADE_Y, z_pos) ;  /* 1 */
  glTexCoord2f (SCALE_TO_TEXTURE_64 (1.0f), 0.0f) ;
  glVertex3f (x_pos, HULL_SHADE_Y, z_pos) ;  /* 2 */
  glTexCoord2f (SCALE_TO_TEXTURE_64 (1.0f), SCALE_TO_TEXTURE_64 (1.0f)) ;
  glVertex3f (x_pos, HULL_SHADE_Y, z_neg) ;  /* 6 */
  glEnd () ;
}





static void RenderHullShade ()
{
  u16 indice_x, indice_z ;
  float world_x, world_z ;
  s8 x_shape_related, y_shape_related, z_shape_related ;
  s8 curr_shape_xorig = current_shape.xorigin ;
  s8 curr_shape_zorig = current_shape.zorigin ;

  for (x_shape_related = 0; x_shape_related <  current_shape.xsize;
       x_shape_related++) {
    for (z_shape_related = 0; z_shape_related <  current_shape.zsize;
	 z_shape_related++) {
      /* Let's check if there is a cube along the Y axis in the shape. */
      for (y_shape_related = 0; y_shape_related <  current_shape.ysize;
	   y_shape_related++) {
	if (current_shape.data[x_shape_related][y_shape_related]
		              [z_shape_related] != Empty_Cell) {
	  /* Render the shade. */
	  indice_x = curr_shape_xorig + x_shape_related ;
	  indice_z = curr_shape_zorig + z_shape_related ;
	  world_x =
            (((float) indice_x) - (((float) (MCUBEDIM - 1)) / 2.0f)) *
	    CUBE_EDGE_SIZE ;
	  world_z =
            - (((float) indice_z) - (((float) (MCUBEDIM - 1)) / 2.0f)) *
	    CUBE_EDGE_SIZE ;
	  ShadedTopCubeFace ((CUBE_EDGE_SIZE / 2.0f), world_x, world_z) ;
	  /* Stop looking along the Y axis. */
	  break ;
	}  /* End of if (current_shape.data[x_shape_related]... */
      }    /* End of for (y_shape_related = 0; ... */
    }      /* End of for (z_shape_related = 0; ... */
  }        /* End of for (x_shape_related = 0; ... */
}





static void RenderMetaCube (float deletion_edge_size)
{
  u16 indice_x, indice_y, indice_z ;
  float world_x, world_y, world_z ;
  bool need_shade ;
  enum cell_type tmp_cell ;
  float edge_size ;
  s8 x_shape_related, y_shape_related, z_shape_related, ygrec ;
  /* To prevent too many dereferenciations in loops. */
  s8 curr_shape_xorig = current_shape.xorigin ;
  s8 curr_shape_yorig = current_shape.yorigin ;
  s8 curr_shape_zorig = current_shape.zorigin ;

  /* Let's render the metacube at the origine. From Y bottom-top, */
  /* X left-right and Z front-back.                               */
  for (indice_y = 0; indice_y < MCUBEDIM; indice_y++) {
    for (indice_x = 0; indice_x < MCUBEDIM; indice_x++) {
      for (indice_z = 0; indice_z < MCUBEDIM; indice_z++) {
	tmp_cell =
	  global_current_metacube->cells[indice_x][indice_y][indice_z] ;
	if (tmp_cell != Empty_Cell) {
	  /* There is a cube in the metacube at this position. */
	  /* Now change the default size of the cube if needed. */
	  if (tmp_cell == Deletion_Marked_Cell)
	    edge_size = deletion_edge_size  / 2.0f ;
	  else edge_size = (CUBE_EDGE_SIZE / 2.0f) ;
	  need_shade = false ;
	  /* Now, check if we need to shade the */
	  /* upper face of the current cube.    */
	  x_shape_related = indice_x - curr_shape_xorig ;
	  y_shape_related = indice_y - curr_shape_yorig ;
	  z_shape_related = indice_z - curr_shape_zorig ;
	  for (ygrec = 0; ygrec < current_shape.ysize; ygrec++) {
	    if ((x_shape_related >= 0) &&
		(x_shape_related < current_shape.xsize)
		&&
		(z_shape_related >= 0) &&
		(z_shape_related < current_shape.zsize)
		&&
		(current_shape.data[x_shape_related]
		                   [ygrec]
		                   [z_shape_related] != Empty_Cell)) {
	      need_shade = true ;
	      break ;
	    }
	  }
	  world_x =
            (((float) indice_x) - (((float) (MCUBEDIM - 1)) / 2.0f)) *
	    CUBE_EDGE_SIZE ;
	  world_y =
            (((float) indice_y) - (((float) (MCUBEDIM - 1)) / 2.0f)) *
	    CUBE_EDGE_SIZE ;
	  world_z =
            - (((float) indice_z) - (((float) (MCUBEDIM - 1)) / 2.0f)) *
	    CUBE_EDGE_SIZE ;
	  RenderHomebrewCube (edge_size,
			      world_x, world_y, world_z, need_shade) ;
	}
      }
    }
  }
}




/* ******************************************* Inlining ! *************** */
#define COMMON_RENDERING_PRELUDE \
    glReset () ; \
    glLoadIdentity () ; \
    gluPerspective (FOV, 256.0f / 192.0f, 0.2f, 20.f) ; \
    RenderCamera (&global_camera) ; \
    \
    glMatrixMode (GL_TEXTURE) ; \
    glIdentity () ; \
    \
    glMatrixMode (GL_MODELVIEW) ; \
    glMaterialf (GL_AMBIENT_AND_DIFFUSE, RGB15 (15, 15, 15)) ; \
    glMaterialf (GL_SPECULAR, BIT (15) | RGB15 (4, 4, 4)) ; \
    glMaterialf (GL_EMISSION, RGB15 (5, 5, 5)) ; \
    glMaterialShinyness () ; \
    \
    glLight (0, RGB15 (31, 31, 31), floattov10 (1.0), floattov10 (0.0), \
	     floattov10 (1.0)) ; \
    glPolyFmt (POLY_ALPHA (31) | POLY_CULL_BACK | POLY_FORMAT_LIGHT0)
/* ********************************************************************* */





#define SOFT_ROTATION_STEP (2)
void DoSoftRotation (enum soft_rotation rotation)
{
  short i ;
  float rx = 0.0f, ry = 0.0f, rz = 0.0f ;
  float alpha = 0.0f ;
  float dalpha = SOFT_ROTATION_STEP ;

  switch (rotation) {
  case X_Clocwise:
    rx = 1.0f ;
    ry = 0.0f ;
    rz = 0.0f ;
    dalpha = (float) SOFT_ROTATION_STEP ;
    break ;
  case X_CounterClocwise:
    rx = 1.0f ;
    ry = 0.0f ;
    rz = 0.0f ;
    dalpha = (float) (- SOFT_ROTATION_STEP) ;
    break ;
  case Y_Clocwise:
    rx = 0.0f ;
    ry = 1.0f ;
    rz = 0.0f ;
    dalpha = (float) SOFT_ROTATION_STEP ;
    break ;
  case Y_CounterClocwise:
    rx = 0.0f ;
    ry = 1.0f ;
    rz = 0.0f ;
    dalpha = (float) (- SOFT_ROTATION_STEP) ;
    break ;

  case Z_Clocwise:
    rx = 0.0f ;
    ry = 0.0f ;
    rz = 1.0f ;
    dalpha = (float) (SOFT_ROTATION_STEP) ;
    break ;
  case Z_CounterClocwise:
    rx = 0.0f ;
    ry = 0.0f ;
    rz = 1.0f ;
    dalpha = (float) (-SOFT_ROTATION_STEP) ;
    break ;

  }

   playGenericSound (g_wav_rotate, WAV_ROTATE_LENGTH * sizeof (u32)) ;

  for (i = 0; i < 90 / SOFT_ROTATION_STEP; i++) {
    COMMON_RENDERING_PRELUDE ;

    glPushMatrix () ;

    glBindTexture (GL_TEXTURE_2D, texture_ids[DEEP_SPACE_TEXTURE_ID]) ;
    RenderDeepSpace () ;

    glBindTexture (GL_TEXTURE_2D, texture_ids[METACUBE_CUBE_TEXTURE_ID]) ;
    RenderShape (&current_shape) ;

    glBindTexture
      (GL_TEXTURE_2D, texture_ids[FLIPPED_CUBE_TEXTURE_SHADED_ID]) ;
    RenderHullShade () ;

    glRotatef (alpha, rx, ry, rz) ;
    glBindTexture (GL_TEXTURE_2D, texture_ids[FLIPPED_CUBE_TEXTURE_ID]) ;
    RenderHomebrewCenteredFlippedHalfCube
      (((MCUBEDIM / 2.0f) * CUBE_EDGE_SIZE) + 0.15f) ;
 
    glBindTexture (GL_TEXTURE_2D, texture_ids[METACUBE_CUBE_TEXTURE_ID]) ;
    RenderMetaCube (CUBE_EDGE_SIZE) ;   /* Parameter will not be used. */

    glPopMatrix (1) ;
    glFlush () ;
    swiWaitForVBlank () ;
    alpha = alpha + dalpha ;
  }
}





void DoSoftDescent (s8 downto_y)
{
  while (current_shape.yorigin > downto_y) {
    current_shape.yorigin-- ;

    COMMON_RENDERING_PRELUDE ;

    glPushMatrix () ;

    glBindTexture (GL_TEXTURE_2D, texture_ids[DEEP_SPACE_TEXTURE_ID]) ;
    RenderDeepSpace () ;

    glBindTexture (GL_TEXTURE_2D, texture_ids[METACUBE_CUBE_TEXTURE_ID]) ;
    RenderShape (&current_shape) ;

    glBindTexture
      (GL_TEXTURE_2D, texture_ids[FLIPPED_CUBE_TEXTURE_SHADED_ID]) ;
    RenderHullShade () ;

    glBindTexture (GL_TEXTURE_2D, texture_ids[FLIPPED_CUBE_TEXTURE_ID]) ;
    RenderHomebrewCenteredFlippedHalfCube
      (((MCUBEDIM / 2.0f) * CUBE_EDGE_SIZE) + 0.15f) ;
 
    glBindTexture (GL_TEXTURE_2D, texture_ids[METACUBE_CUBE_TEXTURE_ID]) ;
    RenderMetaCube (CUBE_EDGE_SIZE) ;  /* Parameter will not be used. */

    glPopMatrix (1) ;
    glFlush () ;
    swiWaitForVBlank () ;
    swiDelay (700000) ;
  }
}




#define ANIM_NB_FRAMES 10
void ShowDestructions ()
{
  u8 i ;
  float cube_edge = CUBE_EDGE_SIZE ;

  playGenericSound (g_wav_disapear, WAV_DISAPEAR_LENGTH * sizeof (u32)) ;
  for (i = 0; i < ANIM_NB_FRAMES; i++) {
    COMMON_RENDERING_PRELUDE ;

    glPushMatrix () ;

    glBindTexture (GL_TEXTURE_2D, texture_ids[DEEP_SPACE_TEXTURE_ID]) ;
    RenderDeepSpace () ;

    //glBindTexture (GL_TEXTURE_2D, texture_ids[METACUBE_CUBE_TEXTURE_ID]) ;
    //RenderShape (&current_shape) ;

    glBindTexture
      (GL_TEXTURE_2D, texture_ids[FLIPPED_CUBE_TEXTURE_SHADED_ID]) ;
    RenderHullShade () ;

    glBindTexture (GL_TEXTURE_2D, texture_ids[FLIPPED_CUBE_TEXTURE_ID]) ;
    RenderHomebrewCenteredFlippedHalfCube
      (((MCUBEDIM / 2.0f) * CUBE_EDGE_SIZE) + 0.15f) ;
 
    glBindTexture (GL_TEXTURE_2D, texture_ids[METACUBE_CUBE_TEXTURE_ID]) ;
    RenderMetaCube (cube_edge) ;  /* Parameter WILL BE used here. */
    cube_edge = cube_edge - (CUBE_EDGE_SIZE / ((float) ANIM_NB_FRAMES)) ;

    glPopMatrix (1) ;
    glFlush () ;
    swiWaitForVBlank () ;
    swiDelay (800000) ;
  }
}


void NotifyBadDrop ()
{
  s16 alpha ;

  playGenericSound (g_wav_dont_fit, WAV_DONT_FIT_LENGTH * sizeof (u32)) ;

  for (alpha = 0; alpha < 360; alpha += 20) {
    COMMON_RENDERING_PRELUDE ;

    glBindTexture (GL_TEXTURE_2D, texture_ids[DEEP_SPACE_TEXTURE_ID]) ;
    RenderDeepSpace () ;

    glBindTexture (GL_TEXTURE_2D, texture_ids[METACUBE_CUBE_TEXTURE_ID]) ;
    /* Rotate the shape... */
    glPushMatrix () ;
    glRotatef (alpha, 0.0f, 0.0f, 1.0f) ;
    RenderShape (&current_shape) ;
    glPopMatrix (1) ;

    glBindTexture
      (GL_TEXTURE_2D, texture_ids[FLIPPED_CUBE_TEXTURE_SHADED_ID]) ;
    RenderHullShade () ;

    glBindTexture (GL_TEXTURE_2D, texture_ids[FLIPPED_CUBE_TEXTURE_ID]) ;
    RenderHomebrewCenteredFlippedHalfCube
      (((MCUBEDIM / 2.0f) * CUBE_EDGE_SIZE) + 0.15f) ;
 
    glBindTexture (GL_TEXTURE_2D, texture_ids[METACUBE_CUBE_TEXTURE_ID]) ;
    RenderMetaCube (CUBE_EDGE_SIZE) ;  /* Parameter will not be used. */

    glPopMatrix (1) ;
    glFlush () ;
    swiWaitForVBlank () ;
  }
}


void DoRegularRendering ()
{
  COMMON_RENDERING_PRELUDE ;

  glPushMatrix () ;

  glBindTexture (GL_TEXTURE_2D, texture_ids[DEEP_SPACE_TEXTURE_ID]) ;
  RenderDeepSpace () ;

  glBindTexture (GL_TEXTURE_2D, texture_ids[METACUBE_CUBE_TEXTURE_ID]) ;
  RenderShape (&current_shape) ;

  glBindTexture (GL_TEXTURE_2D, texture_ids[FLIPPED_CUBE_TEXTURE_ID]) ;
  RenderHomebrewCenteredFlippedHalfCube
    (((MCUBEDIM / 2.0f) * CUBE_EDGE_SIZE) + 0.15f) ;

  glBindTexture (GL_TEXTURE_2D, texture_ids[METACUBE_CUBE_TEXTURE_ID]) ;
  RenderMetaCube (CUBE_EDGE_SIZE) ;  /* Parameter will not be used. */

  glBindTexture (GL_TEXTURE_2D, texture_ids[FLIPPED_CUBE_TEXTURE_SHADED_ID]) ;
  RenderHullShade () ;

  glPopMatrix (1) ;
  glFlush () ;
  swiWaitForVBlank () ;
}



void AnimateTouchOnSprite_0 (s16 at_x, s16 at_y)
{
  short i ;

  playGenericSound
    (g_wav_bubble_pop, WAV_BUBBLE_POP_LENGTH * sizeof (u32)) ;

  at_x = max (0, at_x - 16) ;
  at_y = max (0, at_y - 16) ;

  global_sprites_copy[0].attribute[0] =
    ATTR0_COLOR_256 | ATTR0_SQUARE | (at_y & 0x00FF) ;
  global_sprites_copy[0].attribute[1] = ATTR1_SIZE_32 | (at_x & 0x01FF) ;
  global_sprites_copy[0].attribute[3] = 0 ;

  for (i = 0; i < NB_BUBBLES_FRAMES ; i++) {
    global_sprites_copy[0].attribute[2] =
      (FIRST_BUBBLES_TILE_INDEX + (i * NB_TILES_PER_BUBBLES_FRAME)) * 2 ;
    swiWaitForVBlank () ;
    swiWaitForVBlank () ;
    swiWaitForVBlank () ;
    memcpy
      ((void*) OAM_SUB, (void*) global_sprites_copy,
       128 * sizeof (struct sSpriteEntry)) ;
  }
  global_sprites_copy[0].attribute[0] = ATTR0_DISABLED ;
  memcpy
    ((void*) OAM_SUB, (void*) global_sprites_copy,
     128 * sizeof (struct sSpriteEntry)) ;
}



void PrepareStylusOnSprite_1 ()
{
  /* Prepare the sprite 1. */
  global_sprites_copy[1].attribute[0] =
    ATTR0_COLOR_256 | ATTR0_SQUARE | 160 ;
  global_sprites_copy[1].attribute[1] = ATTR1_SIZE_32 | 230 ;
  global_sprites_copy[1].attribute[3] = 0 ;
}


void HideStylusOnSprite_1 ()
{
  global_sprites_copy[1].attribute[0] = ATTR0_DISABLED ;
  swiWaitForVBlank () ;
  memcpy
    ((void*) OAM_SUB, (void*) global_sprites_copy,
     128 * sizeof (struct sSpriteEntry)) ;
}


void AnimateStylusOnSprite_1 ()
{
  static u32 i ;

  /* Animate each 1/2 second. */
  global_sprites_copy[1].attribute[2] =
    (FIRST_STYLUS_TILE_INDEX + ((i >> 3) * (NB_TILES_PER_STYLUS_FRAME * 2))) ;
  swiWaitForVBlank () ;
  memcpy
    ((void*) OAM_SUB, (void*) global_sprites_copy,
     128 * sizeof (struct sSpriteEntry)) ;
  i = (i + 1) % (NB_STYLUS_FRAMES << 3) ;
}



void AnimateSleepingSmilesAndLoopForever ()
{
  u16 i, j ;

  /* Initialisation. */
  for (i = 0; i < 4; i++) {
    global_sprites_copy[i].attribute[0] = ATTR0_COLOR_256 | ATTR0_SQUARE | 24 ;
  global_sprites_copy[i].attribute[1] = ATTR1_SIZE_32 | (8 + (i * 40)) ;
  global_sprites_copy[i].attribute[3] = 0 ;
  }

  for (i = 0;;i++) {
    /* Just update each sprite frame. */
    for (j = 0; j < 4; j++) {
      global_sprites_copy[j].attribute[2] =
	(FIRST_SLEEP_ANIM_TILE_INDEX + 
	 (((i + j) % NB_SLEEP_ANIM_FRAMES) * (NB_TILES_PER_SLEEP_FRAME * 2))) ;
    }

    /* Final transfer to OAM. */
    swiWaitForVBlank () ;
    memcpy
      ((void*) OAM_SUB, (void*) global_sprites_copy,
       128 * sizeof (struct sSpriteEntry)) ;
    swiDelay (1000000) ;
  }
}



bool ConfirScoreErasing ()
{
  u16 keys_held, keys_down ;
  touchPosition stylus_xy ;
  bool confirm = false ;

  global_sprites_copy[1].attribute[0] =
    ATTR0_COLOR_256 | ATTR0_WIDE | SCORE_ERASING_POPUP_Y ;
  global_sprites_copy[1].attribute[1] = ATTR1_SIZE_32 | SCORE_ERASING_POPUP_X ;
  global_sprites_copy[1].attribute[2] = FIRST_YES_NO_TILE_INDEX ;
  global_sprites_copy[1].attribute[3] = 0 ;
  swiWaitForVBlank () ;
  memcpy
    ((void*) OAM_SUB, (void*) global_sprites_copy,
     128 * sizeof (struct sSpriteEntry)) ;

  for (;;) {
    scanKeys () ;
    stylus_xy = touchReadXY () ;
    keys_held = keysHeld () ;
    keys_down = keysDown () ;

    if (keys_down & (KEY_B)) {
      confirm = false ;
      break ;
      }

    if (keys_held & KEY_TOUCH) {
      u16 x = stylus_xy.px ;
      u16 y = stylus_xy.py ;

      /* Check for buttons at the vertical hit location. */
      if ((y > SCORE_ERASING_POPUP_Y) &&
	  (y <= SCORE_ERASING_POPUP_Y + SCORE_ERASING_BUTTONS_HEIGHT)) {
	/* Check for buttons at the horizontal hit location. */
	if ((x >= SCORE_ERASING_YES_X) &&
	    (x < SCORE_ERASING_YES_X + SCORE_ERASING_BUTTONS_WIDTH)) {
	  /* Really erase. */
	  AnimateTouchOnSprite_0 (x, y) ;
	  confirm = true ;
	  break ;
	}
	if ((x >= SCORE_ERASING_NO_X) &&
	    (x < SCORE_ERASING_NO_X + SCORE_ERASING_BUTTONS_WIDTH)) {
	  /* Do not erase. */
	  AnimateTouchOnSprite_0 (x, y) ;
	  confirm = false ;
	  break ;
	}
      }         /* End of if ((y > SCORE_ERASING_POPUP_Y) && ... */
    }           /* End of if (keys_held & KEY_TOUCH). */
  }             /* End of for (;;). */

  HideAllSprites () ;
  return (confirm) ;
}




void FlashBottomScreen ()
{
  u16 old_blending_mode ;
  short i ;

  /* Petite anim. */
  old_blending_mode = SUB_BLEND_CR ;
  SUB_BLEND_Y = 0 ;
  SUB_BLEND_CR = BLEND_FADE_WHITE | BLEND_SRC_BG0 ;
  for (i = 0; i < 16; i++) {
    swiWaitForVBlank () ;
    SUB_BLEND_Y = (u16) i ;
    BG0_X0 = ((s16) ((rand ()) % 8)) - 4 ;
    BG0_Y0 = ((s16) ((rand ()) % 8)) - 4 ;
  }
  for (i = 15; i >= 0; i--) {
    swiWaitForVBlank () ;
    SUB_BLEND_Y = (u16) i ;
    BG0_X0 = ((s16) ((rand ()) % 8)) - 4 ;
    BG0_Y0 = ((s16) ((rand ()) % 8)) - 4 ;
  }

  BG0_X0 = 0 ;
  BG0_Y0 = 0 ;
  SUB_BLEND_CR = old_blending_mode ;
}
