#include <nds.h>
#include "gfx_engine.h"
#include "homebrew_cube.h"

/*

	 -+-   5-------------6  ++-
	      /|            /|
	     / |           / |
	    /  |          /  |
	   /   |         /   |
  -++     1----+---------2   |  +++
	  |    |         |   |
    ---   |    4---------+---7 +--
	  |   /          |   /
	  |  /           |  /
	  | /            | /
	  |/             |/
  --+     0--------------3   +-+

*/



void RenderHomebrewCube (float radius, float cx, float cy, float cz,
			 bool upper_shaded)
{
  float x_neg = -radius + cx ;
  float x_pos = radius + cx ;
  float y_neg = -radius + cy ;
  float y_pos = radius + cy ;
  float z_neg = -radius + cz ;
  float z_pos = radius + cz ;

  /* Render the upper side apart to be able to change the texture. */
  /* If shaded needed, then just enable the "shaded" texture. */
  /* Remind that one can't change texture binding between glBegin and glEnd. */
  if (upper_shaded)
    glBindTexture
    (GL_TEXTURE_2D, texture_ids[METACUBE_CUBE_TEXTURE_SHADED_ID]) ;
  glBegin (GL_QUADS) ;
  /* Top Face: points up on y. */
  glNormal3f (0.0f, 1.0f, 0.0f) ;
  glTexCoord2f (0.0f, SCALE_TO_TEXTURE_64 (1.0f)) ;
  glVertex3f (x_neg, y_pos, z_neg) ;  /* 5 */
  glTexCoord2f (0.0f, 0.0f) ;
  glVertex3f (x_neg, y_pos, z_pos) ;  /* 1 */
  glTexCoord2f (SCALE_TO_TEXTURE_64 (1.0f), 0.0f) ;
  glVertex3f (x_pos, y_pos, z_pos) ;  /* 2 */
  glTexCoord2f (SCALE_TO_TEXTURE_64 (1.0f), SCALE_TO_TEXTURE_64 (1.0f)) ;
  glVertex3f (x_pos, y_pos, z_neg) ;  /* 6 */
  glEnd () ;
  /* If shaded was needed, then restore the regular texture. */
  if (upper_shaded)
    glBindTexture
      (GL_TEXTURE_2D, texture_ids[METACUBE_CUBE_TEXTURE_ID]) ;

  glBegin (GL_QUADS) ;
  /* Front face: points out of screen on z. */
  glNormal3f (0.0f, 0.0f, 1.0f) ;
  glTexCoord2f (0.0f, 0.0f) ;
  glVertex3f (x_neg, y_neg, z_pos) ;  /* 0 */
  glTexCoord2f (SCALE_TO_TEXTURE_64 (1.0f), 0.0f) ;
  glVertex3f (x_pos, y_neg, z_pos) ;  /* 3 */
  glTexCoord2f (SCALE_TO_TEXTURE_64 (1.0f), SCALE_TO_TEXTURE_64 (1.0f)) ;
  glVertex3f (x_pos, y_pos, z_pos) ;  /* 2 */
  glTexCoord2f (0.0f, SCALE_TO_TEXTURE_64 (1.0f)) ;
  glVertex3f (x_neg, y_pos, z_pos) ;  /* 1 */

  /* Back face: points into the screen on z. */
  glNormal3f (0.0f, 0.0f, -1.0f) ;
  glTexCoord2f (SCALE_TO_TEXTURE_64 (1.0f), 0.0f) ;
  glVertex3f (x_neg, y_neg, z_neg) ;  /* 4 */
  glTexCoord2f (SCALE_TO_TEXTURE_64 (1.0f), SCALE_TO_TEXTURE_64 (1.0f)) ;
  glVertex3f (x_neg, y_pos, z_neg) ;  /* 5 */
  glTexCoord2f (0.0f, SCALE_TO_TEXTURE_64 (1.0f)) ;
  glVertex3f (x_pos, y_pos, z_neg) ;  /* 6 */
  glTexCoord2f (0.0f, 0.0f) ;
  glVertex3f (x_pos, y_neg, z_neg) ;  /* 7 */

  /* Bottom Face: points down on y. */
  glNormal3f (0.0f, -1.0f, 0.0f) ;
  glTexCoord2f (1.0f, SCALE_TO_TEXTURE_64 (1.0f)) ;
  glVertex3f (x_neg, y_neg, z_neg) ;  /* 4 */
  glTexCoord2f (0.0f, SCALE_TO_TEXTURE_64 (1.0f)) ;
  glVertex3f (x_pos, y_neg, z_neg) ;  /* 7 */
  glTexCoord2f (0.0f, 0.0f) ;
  glVertex3f (x_pos, y_neg, z_pos) ;  /* 3 */
  glTexCoord2f (SCALE_TO_TEXTURE_64 (1.0f), 0.0f) ;
  glVertex3f (x_neg, y_neg, z_pos) ;  /* 0 */

  /* Right face: points right on x. */
  glNormal3f (1.0f, 0.0f, 0.0f) ;
  glTexCoord2f (SCALE_TO_TEXTURE_64 (1.0f), 0.0f) ;
  glVertex3f (x_pos, y_neg, z_neg) ;  /* 7 */
  glTexCoord2f (SCALE_TO_TEXTURE_64 (1.0f), SCALE_TO_TEXTURE_64 (1.0f)) ;
  glVertex3f (x_pos, y_pos, z_neg) ;  /* 6 */
  glTexCoord2f (0.0f, SCALE_TO_TEXTURE_64 (1.0f)) ;
  glVertex3f (x_pos, y_pos, z_pos) ;  /* 2 */
  glTexCoord2f (0.0f, 0.0f) ;
  glVertex3f (x_pos, y_neg, z_pos) ;  /* 3 */

  /* Left Face: points left on x. */
  glNormal3f (-1.0f, 0.0f, 0.0f) ;
  glTexCoord2f (0.0f, 0.0f) ;
  glVertex3f (x_neg, y_neg, z_neg) ;  /* 4 */
  glTexCoord2f (SCALE_TO_TEXTURE_64 (1.0f), 0.0f) ;
  glVertex3f (x_neg, y_neg, z_pos) ;  /* 0 */
  glTexCoord2f (SCALE_TO_TEXTURE_64 (1.0f), SCALE_TO_TEXTURE_64 (1.0f)) ;
  glVertex3f (x_neg, y_pos, z_pos) ;  /* 1 */
  glTexCoord2f (0.0f, SCALE_TO_TEXTURE_64 (1.0f)) ;
  glVertex3f (x_neg, y_pos, z_neg) ;  /* 5 */
  glEnd () ;
}



// TODO: Flipp faces and normals !
void RenderHomebrewCenteredFlippedHalfCube (float radius)
{

  glBegin (GL_QUADS) ;

  /* Back face: points out of the screen on z. */
  glNormal3f (0.0f, 0.0f, 1.0f) ;
  glTexCoord2f (1.0f, 0.0f) ;
  glVertex3f (-radius, -radius, -radius) ;  /* 4 */
  glTexCoord2f (0.0f, 0.0f) ;
  glVertex3f (radius, -radius, -radius) ;  /* 7 */
  glTexCoord2f (0.0f, 1.0f) ;
  glVertex3f (radius,  radius, -radius) ;  /* 6 */
  glTexCoord2f (1.0f, 1.0f) ;
  glVertex3f (-radius,  radius, -radius) ;  /* 5 */

  /* Bottom Face: points up on y. */
  glNormal3f (0.0f, 1.0f, 0.0f) ;
  glTexCoord2f (1.0f, 1.0f) ;
  glVertex3f (-radius, -radius, -radius) ;  /* 4 */
  glTexCoord2f (1.0f, 0.0f) ;
  glVertex3f (-radius, -radius,  radius) ;  /* 0 */
  glTexCoord2f (0.0f, 0.0f) ;
  glVertex3f (radius, -radius,  radius) ;  /* 3 */
  glTexCoord2f (0.0f, 1.0f) ;
  glVertex3f (radius, -radius, -radius) ;  /* 7 */

  /* Left Face: points right on x. */
  glNormal3f (1.0f, 0.0f, 0.0f) ;
  glTexCoord2f (0.0f, 0.0f) ;
  glVertex3f (-radius, -radius, -radius) ; /* 4 */
  glTexCoord2f (0.0f, 1.0f) ;
  glVertex3f (-radius,  radius, -radius) ; /* 5 */
  glTexCoord2f (1.0f, 1.0f) ;
  glVertex3f (-radius,  radius,  radius) ; /* 1 */
  glTexCoord2f (1.0f, 0.0f) ;
  glVertex3f (-radius, -radius,  radius) ; /* 0 */

  glEnd () ;
}
