#include <math.h>
#include "mathematics.h"


/* Supports overlapping. */
void VectorsAdd (struct point *in1, struct point *in2, struct point *out)
{
  out->x = in1->x + in2->x ;
  out->y = in1->y + in2->y ;
  out->z = in1->z + in2->z ;
}


/* Supports overlapping. */
void VectorScale (struct point *in, float scale, struct point *out)
{
  out->x = in->x * scale ;
  out->y = in->y * scale ;
  out->z = in->z * scale ;
}


/* Supports overlapping. */
static void NormalizeVector (struct point *in, struct point *out)
{
  float l = sqrtf (in->x * in->x + in->y * in->y + in->z * in->z) ;

  if (l == 0.0f) {
    out->x = out->y = out->z = 0.0f ;
    return ;
  }

  out->x = in->x / l ;
  out->y = in->y / l ;
  out->z = in->z / l ;
}




/* Supports overlapping. */
void RotateAroundArbitrary (struct point *v, struct point *axis, float angle,
			    struct point *out)
{
  float c, s, t, l ;
  struct point normed_axis ;
  struct point tmp ;

  if ((v->x == 0.0f) && (v->y == 0.0f) && (v->z == 0.0f)) {
    out->x = v->x ;
    out->y = v->y ;
    out->z = v->z ;
    return ;
  }
      
  NormalizeVector (axis, &normed_axis) ;
      
  // calculate parameters of the rotation matrix
  c = cosf (angle) ;
  s = sinf (angle) ;
  t = 1.0f - c ;
      
  /* Multiply v with rotation matrix. */
  tmp.x =
    (t * normed_axis.x * normed_axis.x + c) * v->x +
    (t * normed_axis.x * normed_axis.y + s * normed_axis.z) * v->y +
    (t * normed_axis.x * normed_axis.z - s * normed_axis.y) * v->z ;
  tmp.y =
    (t * normed_axis.x * normed_axis.y - s * normed_axis.z) * v->x +
    (t * normed_axis.y * normed_axis.y + c) * v->y +
    (t * normed_axis.y * normed_axis.z + s * normed_axis.x) * v->z ;
  tmp.z =
    (t * normed_axis.x * normed_axis.z + s * normed_axis.y) * v->x +
    (t * normed_axis.y * normed_axis.z - s * normed_axis.x) * v->y +
    (t * normed_axis.z * normed_axis.z + c) * v->z ;
      
  NormalizeVector (&tmp, &tmp) ;
  l = sqrtf (v->x * v->x + v->y * v->y + v->z * v->z) ;
  VectorScale (&tmp, l, out) ;
}
