#ifndef __METACUBE_H__
#define __METACUBE_H__


#define MAX_SHAPE_DIM (4)

struct shape {
  s8 xorigin ;
  s8 yorigin ;
  s8 zorigin ;
  u8 xsize ;
  u8 ysize ;
  u8 zsize ;
  u8 weight ;   /* Number of non-empty cells. */
  /*             X              Y              Z      */
  u8 data[MAX_SHAPE_DIM][MAX_SHAPE_DIM][MAX_SHAPE_DIM] ;
};

enum fitting_status { Fit_Outside, Fit_Inside, Fit_Not };

#define IMPOSSIBLE_DROP_Z (-42)

void MakeRandomShape (struct shape *dest, s8 xsize, s8 ysize, s8 zsize) ;
enum fitting_status FitP (struct shape *s) ;
bool TranslateZForwardShape (struct shape *s) ;
bool TranslateZBackwardShape (struct shape *s) ;
bool TranslateXLeftShape (struct shape *s) ;
bool TranslateXRightShape (struct shape *s) ;
s8 DropShapeP (struct shape *s) ;
void MergeShape (struct shape *s) ;


#define MCUBEDIM (5)


enum cell_type {
  Empty_Cell = 0,
  Cube_In_Cell = 1,
  Deletion_Marked_Cell = 2
};

struct metacube {
  /*           X        Y        Z */
  enum cell_type cells[MCUBEDIM][MCUBEDIM][MCUBEDIM] ;
  /*                   Y        Z */
  u8 counts_along_x[MCUBEDIM][MCUBEDIM] ;
  /*                   X        Z */
  u8 counts_along_y[MCUBEDIM][MCUBEDIM] ;
 /*                   X        Y */
  u8 counts_along_z[MCUBEDIM][MCUBEDIM] ;
};

void RotateMetacubeZ90CounterClockwise (struct metacube *src,
					struct metacube *dest) ;
void RotateMetacubeZ90Clockwise (struct metacube *src, struct metacube *dest) ;
void RotateMetacubeX90Clockwise (struct metacube *src, struct metacube *dest) ;
void RotateMetacubeX90CounterClockwise (struct metacube *src, struct metacube *dest) ;
void RotateMetacubeY90Clockwise (struct metacube *src,
				 struct metacube *dest) ;
void RotateMetacubeY90CounterClockwise (struct metacube *src,
					struct metacube *dest) ;
void MakeRandomMetacube (struct metacube *dest) ;
void MakeEmptyMetacube (struct metacube *dest) ;
void CheckDestructions (struct shape *s) ;

#endif
