#include <nds.h>
#include <stdio.h>
#include "gfx_load.h"
#include "global.h"
#include "mainloop.h"
#include "options_bottom_gfx.h"
#include "gfx_engine.h"
#include "persistent.h"
#include "sprites_gfx.h"



static void UpdateKeyboardConfigMarker (enum keyboard_config conf)
{
  u16 *background_video_memory = (u16*) SCREEN_BASE_BLOCK_SUB (0) ;
  u16 tile_on_screen_to_draw, tile_on_screen_to_clear ;
  u16 tile_num_in_marker = OPTIONS_BOTTOM_MARKER_FIRST_TILE_INDEX ;
  u8 x, y ;

  switch (conf) {
  case Keyboard_Config_1:
    for (y = 0; y < MARKERS_HEIGHT_IN_TILES; y++) {
      for (x = 0; x < MARKERS_WIDTH_IN_TILES; x++) {
	tile_on_screen_to_draw =
	  ((KBD_MODE1_BUTTON_TOP_OFFSET + y) * (256 / 8)) +
	  MARKERS_X_DISPLAY_OFFSET + x ;
	tile_on_screen_to_clear =
	  ((KBD_MODE2_BUTTON_TOP_OFFSET + y) * (256 / 8)) +
	  MARKERS_X_DISPLAY_OFFSET + x ;
	/* Draw top marker. */
	background_video_memory[tile_on_screen_to_draw] = tile_num_in_marker ;
	tile_num_in_marker++ ;
	/* Clear bottom marker. */
	background_video_memory[tile_on_screen_to_clear] =
	  OPTIONS_BOTTOM_BLANK_MARKER_FIRST_TILE_INDEX ;
      } /* End of for (x = 0; x < MARKERS_WIDTH_IN_TILES; x++). */
    }   /* End of for (y = 0; y < MARKERS_HEIGHT_IN_TILES; y++). */
    break ;

  case Keyboard_Config_2:
    for (y = 0; y < MARKERS_HEIGHT_IN_TILES; y++) {
      for (x = 0; x < MARKERS_WIDTH_IN_TILES; x++) {
	tile_on_screen_to_draw =
	  ((KBD_MODE2_BUTTON_TOP_OFFSET + y) * (256 / 8)) +
	  MARKERS_X_DISPLAY_OFFSET + x ;
	tile_on_screen_to_clear =
	  ((KBD_MODE1_BUTTON_TOP_OFFSET + y) * (256 / 8)) +
	  MARKERS_X_DISPLAY_OFFSET + x ;
	/* Draw top marker. */
	background_video_memory[tile_on_screen_to_draw] = tile_num_in_marker ;
	tile_num_in_marker++ ;
	/* Clear bottom marker. */
	background_video_memory[tile_on_screen_to_clear] =
	  OPTIONS_BOTTOM_BLANK_MARKER_FIRST_TILE_INDEX ;
      } /* End of for (x = 0; x < MARKERS_WIDTH_IN_TILES; x++). */
    }   /* End of for (y = 0; y < MARKERS_HEIGHT_IN_TILES; y++). */
    break ;
  }
}




static void WaitOptionsInput ()
{
  u16 keys_down, keys_held ;
  touchPosition stylus_xy ;

  for (;;) {
    scanKeys () ;
    stylus_xy = touchReadXY () ;
    keys_held = keysHeld () ;
    keys_down = keysDown () ;
    
    /* B button returns by default. */
    if (keys_down & (KEY_B)) break ;

    if (keys_held & KEY_TOUCH) {
      u16 x = stylus_xy.px ;
      u16 y = stylus_xy.py ;

      /* Check if horizontal hit location is right for kbd mode buttons. */
      if ((x >= KBD_MODE_BUTTONS_X) &&
	  (x < KBD_MODE_BUTTONS_X + KBD_MODE_BUTTONS_WIDTH)) {
	if ((y > KBD_MODE1_BUTTON_Y) && 
	    (y <= KBD_MODE1_BUTTON_Y + KBD_MODE_BUTTONS_HEIGHT)) {
	  /* Switch to keyboard configuration 1. */
	  AnimateTouchOnSprite_0 (x, y) ;
	  SetKeyboardConfig (Keyboard_Config_1) ;
	  UpdateKeyboardConfigMarker (Keyboard_Config_1) ;
	}
	if ((y > KBD_MODE2_BUTTON_Y) && 
	    (y <= KBD_MODE2_BUTTON_Y + KBD_MODE_BUTTONS_HEIGHT)) {
	  /* Switch to keyboard configuration 2. */
	  AnimateTouchOnSprite_0 (x, y) ;
	  SetKeyboardConfig (Keyboard_Config_2) ;
	  UpdateKeyboardConfigMarker (Keyboard_Config_2) ;
	}
      }    /* End of if ((x >= KBD_MODE_BUTTONS_X)... */

      /* Check if horizontal hit location is right for score reset or exit. */
      if ((x >= SCORE_RESET_AND_EXIT_BUTTONS_X) &&
	  (x < SCORE_RESET_AND_EXIT_BUTTONS_X +
	   SCORE_RESET_AND_EXIT_BUTTONS_WIDTH)) {
	/* Check for score reset. */
	if ((y > SCORE_RESET_BUTTON_Y) &&
	    (y <= SCORE_RESET_BUTTON_Y +SCORE_RESET_AND_EXIT_BUTTONS_HEIGHT)) {
	  /* Reset highscore. */
	  AnimateTouchOnSprite_0 (x, y) ;
	  if (ConfirScoreErasing ()) EraseScores () ;
	}
	if ((y > EXIT_BUTTON_Y) &&
	    (y <= EXIT_BUTTON_Y + SCORE_RESET_AND_EXIT_BUTTONS_HEIGHT)) {
	  AnimateTouchOnSprite_0 (x, y) ;
	  break ;
	}  /* End of if ((y > EXIT_BUTTON_Y)... */
      }    /* End of if ((x >= SCORE_RESET_AND_EXIT_BUTTONS_X) ... */

    }      /* End of if (keys_held & KEY_TOUCH) . */
  }

  /* In fact just to save the possibly change keyboard configuration. */
  SaveScores () ;
  return ;
}



void Options ()
{
  s16 i ;

  /* No change on the upper screen. */
  videoSetModeSub (0) ;

  vramSetBankC (VRAM_C_SUB_BG_0x6200000);     // Bg 0x06200000 - 0x06040000

  /* Lower screen. */
  load_palette_tile_map_256
    ((u16*) CHAR_BASE_BLOCK_SUB (1),      // TILES (16ko aligned) (address)
     (u16*) SCREEN_BASE_BLOCK_SUB (0),    // MAP (2ko aligned) (address)
     OPTIONS_BOTTOM_PALETTE_SIZE,
     options_bottom_palette,
     NB_OPTIONS_BOTTOM_TILES,
     options_bottom_tiles,
     OPTIONS_BOTTOM_MAP_WIDTH,
     OPTIONS_BOTTOM_MAP_HEIGHT,
     options_bottom_map,
     Video_sub) ;

  /* Hand marker. */
  load_palette_tile_map_256
    (((u16*) CHAR_BASE_BLOCK_SUB (1)) +
     // TILES (16ko aligned) (address)
     OPTIONS_BOTTOM_MARKER_TILES_LOADING_OFFSET,
     NULL,                               // MAP (2ko aligned) (address)
     0,
     NULL,
     NB_OPTIONS_BOTTOM_MARKER_TILES,
     options_bottom_marker_tiles,
     0,
     0,
     NULL,
     Video_sub) ;

  /* Empty marker. */
  load_palette_tile_map_256
    (((u16*) CHAR_BASE_BLOCK_SUB (1)) +
     // TILES (16ko aligned) (address)
     OPTIONS_BOTTOM_BLANK_MARKER_TILES_LOADING_OFFSET,
     NULL,                               // MAP (2ko aligned) (address)
     0,
     NULL,
     NB_OPTIONS_BOTTOM_BLANK_MARKER_TILES,
     options_bottom_blank_marker_tiles,
     0,
     0,
     NULL,
     Video_sub) ;

    load_sprites_256
      (SPRITES_LOADING_OFFSET,
       NB_SPRITES_TILES,
       sprites_tiles,
       SPRITES_PALETTE_SIZE,
       sprites_palette,
       Video_sub) ;

    HideAllSprites () ;

  SUB_BG0_CR =
      BG_MAP_BASE (0)        // Map (not address).
    | BG_TILE_BASE (1)       // Tiles (not address).
    | BG_PRIORITY_3          // Priorit la plus faible.
    | BG_COLOR_256           // 256 colors.
    | TEXTBG_SIZE_256x256 ;  // 256 x 256 pixels

  /* Initial keyboard config marker refresh. */
  UpdateKeyboardConfigMarker (GetKeyboardConfig ()) ;

  /* Screen appears fading. */
  SUB_BLEND_Y = 15 ;
  SUB_BLEND_CR = BLEND_FADE_BLACK | BLEND_SRC_BG0 ;
  videoSetModeSub (MODE_0_2D | DISPLAY_BG0_ACTIVE|
		   DISPLAY_SPR_1D_LAYOUT | DISPLAY_SPR_ACTIVE) ;

  for (i = 31; i > 0; i--) {
    swiWaitForVBlank () ;
    SUB_BLEND_Y = (u16) (i / 2) ;
  }
  SUB_BLEND_CR = 0 ;

  /* Attente d'une slection de mode de jeu. */
  WaitOptionsInput () ;
  
  /* Screen disappears fading. */
  SUB_BLEND_Y = 0 ;
  SUB_BLEND_CR = BLEND_FADE_BLACK | BLEND_SRC_BG0 ;
  for (i = 0; i < 32; i++) {
    swiWaitForVBlank () ;
    SUB_BLEND_Y = (u16) (i / 2) ;
  }

  SUB_BLEND_CR = 0 ;
  videoSetModeSub (0) ;
}
