#include <nds.h>
#include "gfx_load.h"
#include "pause_bottom_gfx.h"
#include "pause.h"
#include "gfx_engine.h"



static enum pause_action WaitPauseInput ()
{
  u16 keys_down, keys_held ;
  touchPosition stylus_xy ;

  for (;;) {
    scanKeys () ;
    stylus_xy = touchReadXY () ;
    keys_held = keysHeld () ;
    keys_down = keysDown () ;

    /* Default B button to continue. */
    if (keys_down & KEY_B) return (PA_Continue) ;

    if (keys_held & KEY_TOUCH) {
      u16 x = stylus_xy.px ;
      u16 y = stylus_xy.py ;

      /* Check if inside the correct buttons horizontal space. */
      if ((x >= PAUSE_BUTTONS_X) &&
	  (x < (PAUSE_BUTTONS_X + PAUSE_BUTTONS_WIDTH))) {

	/* Now check which button was pressed if any. */
	if ((y >= PAUSE_BUTTON_CONTINUE_Y) &&
	    (y < (PAUSE_BUTTON_CONTINUE_Y + PAUSE_BUTTONS_HEIGHT))) {
	  AnimateTouchOnSprite_0 (x, y) ;
	  return (PA_Continue) ;
	  /* "Continue" button. */
	}   /* End of the "Continue" button. */

	if ((y >= PAUSE_BUTTON_HIBERNATE_Y) &&
	    (y < (PAUSE_BUTTON_HIBERNATE_Y + PAUSE_BUTTONS_HEIGHT))) {
	  /* "Hibernate" button. */
	  AnimateTouchOnSprite_0 (x, y) ;
	  return (PA_Hibernate) ;
	}   /* End of the "Hibernate" button. */

	if ((y >= PAUSE_BUTTON_ABANDON_Y) &&
	    (y < (PAUSE_BUTTON_ABANDON_Y + PAUSE_BUTTONS_HEIGHT))) {
	  /* "Abandon" button. */
	  AnimateTouchOnSprite_0 (x, y) ;
	  return (PA_Abandon) ;
	}   /* End of the "Abandon" button. */

      }     /* End of if ((x >= ...)) */
    }       /* End of if (keys_held & KEY_TOUCH). */
  }         /* End of for (;;). */
}





enum pause_action Pause ()
{
  enum pause_action action ;
  s16 i ;

  /* We don't kill the game info map to prevent refreshing the display. */
  load_palette_tile_map_256
    ((u16*) CHAR_BASE_BLOCK_SUB (2),     // TILES (16ko aligned) (address)
     (u16*) SCREEN_BASE_BLOCK_SUB (1),   // MAP (2ko aligned) (address)
     PAUSE_BOTTOM_PALETTE_SIZE,
     pause_bottom_palette,
     NB_PAUSE_BOTTOM_TILES,
     pause_bottom_tiles,
     PAUSE_BOTTOM_MAP_WIDTH,
     PAUSE_BOTTOM_MAP_HEIGHT,
     pause_bottom_map,
     Video_sub) ;

  SUB_BG1_CR =
      BG_MAP_BASE (1)        // Map (not address).
    | BG_TILE_BASE (2)       // Tiles (not address).
    | BG_PRIORITY_2          // Priority just above the game info.
    | BG_COLOR_256           // 256 colors.
    | TEXTBG_SIZE_256x256 ;  // 256 x 256 pixels

  /* Screen appears fading. */
  SUB_BLEND_Y = 15 ;
  SUB_BLEND_CR = BLEND_FADE_BLACK | BLEND_SRC_BG1 ;
  videoSetModeSub (MODE_0_2D | DISPLAY_BG1_ACTIVE |
		   DISPLAY_SPR_1D_LAYOUT | DISPLAY_SPR_ACTIVE) ;
   
  for (i = 31; i > 0; i--) {
    swiWaitForVBlank () ;
    SUB_BLEND_Y = (u16) (i / 2) ;
  }
  SUB_BLEND_CR = 0 ;

  /* Attente d'une slection de mode de jeu. */
  action = WaitPauseInput () ;

  /* Screen disappears fading. */
  SUB_BLEND_Y = 0 ;
  SUB_BLEND_CR = BLEND_FADE_BLACK | BLEND_SRC_BG1 ;
  for (i = 0; i < 32; i++) {
    swiWaitForVBlank () ;
    SUB_BLEND_Y = (u16) (i / 2) ;
  }

  SUB_BLEND_CR = 0 ;
  videoSetModeSub (0) ;
  return (action) ;
}
