#include <nds.h>
#include <string.h>
#include <stdlib.h>
#include "metacube.h"
#include "global.h"
#include "persistent.h"
#include "mainloop.h"
#include "time.h"



/* Pour viter d'allouer un gros truc sur la pile. */
static struct save_data tmp_data ;
struct highscores global_highscores = {
  { { 0, 0, EMPTY_PLAYER_NAME },
    { 0, 0, EMPTY_PLAYER_NAME },
    { 0, 0, EMPTY_PLAYER_NAME },
    { 0, 0, EMPTY_PLAYER_NAME },
    { 0, 0, EMPTY_PLAYER_NAME } },
  { { 0, 0, EMPTY_PLAYER_NAME },
    { 0, 0, EMPTY_PLAYER_NAME },
    { 0, 0, EMPTY_PLAYER_NAME },
    { 0, 0, EMPTY_PLAYER_NAME },
    { 0, 0, EMPTY_PLAYER_NAME } },
  { { 0, 0, EMPTY_PLAYER_NAME },
    { 0, 0, EMPTY_PLAYER_NAME },
    { 0, 0, EMPTY_PLAYER_NAME },
    { 0, 0, EMPTY_PLAYER_NAME },
    { 0, 0, EMPTY_PLAYER_NAME } },
  { { 0, 0, EMPTY_PLAYER_NAME },
    { 0, 0, EMPTY_PLAYER_NAME },
    { 0, 0, EMPTY_PLAYER_NAME },
    { 0, 0, EMPTY_PLAYER_NAME },
    { 0, 0, EMPTY_PLAYER_NAME } }
};



static void bytes_memcpy (u8* dest, u8 const *src, u32 size)
{
  while (size--) *dest++ = *src++ ;
}





void SaveScores ()
{
  u16 i ;

  /* It is now safe to write GBA Cartridge memory. */
  WAIT_CR &= ~0x0880 ;

  tmp_data.magic_tag = MAGICNUMBER ;
  tmp_data.seed = rand () ;
  /* By the way, save the keyboard config. */
  tmp_data.keyboard_config = GetKeyboardConfig () ;

  /* Copy scores in temporary memory. */
  for (i = 0; i < NB_HIGHSCORES; i++) {
    tmp_data.highscores.endless[i].score =
      global_highscores.endless[i].score ;
    tmp_data.highscores.endless[i].lines =
      global_highscores.endless[i].lines ;
    strcpy (tmp_data.highscores.endless[i].player,
	    global_highscores.endless[i].player) ;

    tmp_data.highscores.game_5mn[i].score =
      global_highscores.game_5mn[i].score ;
    tmp_data.highscores.game_5mn[i].lines =
      global_highscores.game_5mn[i].lines ;
    strcpy (tmp_data.highscores.game_5mn[i].player,
	    global_highscores.game_5mn[i].player) ;

    tmp_data.highscores.game_10mn[i].score =
      global_highscores.game_10mn[i].score ;
    tmp_data.highscores.game_10mn[i].lines =
      global_highscores.game_10mn[i].lines ;
    strcpy (tmp_data.highscores.game_10mn[i].player,
	    global_highscores.game_10mn[i].player) ;

    tmp_data.highscores.game_15mn[i].score =
      global_highscores.game_15mn[i].score ;
    tmp_data.highscores.game_15mn[i].lines =
      global_highscores.game_15mn[i].lines ;
    strcpy (tmp_data.highscores.game_15mn[i].player,
	    global_highscores.game_15mn[i].player) ;
  }

  /* And now, write the SRAM. */
  bytes_memcpy ((u8*) SRAM, (u8 const*) &tmp_data, sizeof (struct save_data)) ;
}



void EraseScores ()
{
  u16 i ;
  const char empty_player_name[MAX_PLAYER_NAME_LENGTH] = EMPTY_PLAYER_NAME ;

  for (i = 0; i < NB_HIGHSCORES; i++) {
    global_highscores.endless[i].score = 0 ;
    global_highscores.endless[i].lines = 0 ;
    strcpy (global_highscores.endless[i].player, empty_player_name) ;

    global_highscores.game_5mn[i].score = 0 ;
    global_highscores.game_5mn[i].lines = 0 ;
    strcpy (global_highscores.game_5mn[i].player, empty_player_name) ;

    global_highscores.game_10mn[i].score = 0 ;
    global_highscores.game_10mn[i].lines = 0 ;
    strcpy (global_highscores.game_10mn[i].player, empty_player_name) ;

    global_highscores.game_15mn[i].score = 0 ;
    global_highscores.game_15mn[i].lines = 0 ;
    strcpy (global_highscores.game_15mn[i].player, empty_player_name) ;
  }

  SaveScores () ;
}




bool ReadScores ()
{
  u16 i ;

  /* It is now safe to read GBA Cartridge memory. */
  WAIT_CR &= ~0x0880 ;
  bytes_memcpy ((u8*) &tmp_data, (u8 const*) SRAM, sizeof (struct save_data)) ;

  /* Ensure there is really some relevant data. */
  if (tmp_data.magic_tag != MAGICNUMBER) return (false) ;

  /* Use the last saved seed to re-init the randomizer. */
  srand (tmp_data.seed) ;
  /* By the way, get the saved keyboard config. */
  SetKeyboardConfig (tmp_data.keyboard_config) ;
  for (i = 0; i < NB_HIGHSCORES; i++) {
    global_highscores.endless[i].score =
      tmp_data.highscores.endless[i].score ;
    global_highscores.endless[i].lines =
      tmp_data.highscores.endless[i].lines ;
    strcpy (global_highscores.endless[i].player,
	    tmp_data.highscores.endless[i].player) ;

    global_highscores.game_5mn[i].score =
      tmp_data.highscores.game_5mn[i].score ;
    global_highscores.game_5mn[i].lines =
      tmp_data.highscores.game_5mn[i].lines ;
    strcpy (global_highscores.game_5mn[i].player, 
	    tmp_data.highscores.game_5mn[i].player) ;

    global_highscores.game_10mn[i].score =
      tmp_data.highscores.game_10mn[i].score ;
    global_highscores.game_10mn[i].lines =
      tmp_data.highscores.game_10mn[i].lines ;
    strcpy (global_highscores.game_10mn[i].player, 
	    tmp_data.highscores.game_10mn[i].player) ;

    global_highscores.game_15mn[i].score =
      tmp_data.highscores.game_15mn[i].score ;
    global_highscores.game_15mn[i].lines =
      tmp_data.highscores.game_15mn[i].lines ;
    strcpy (global_highscores.game_15mn[i].player, 
	    tmp_data.highscores.game_15mn[i].player) ;
  }

  return (true) ;
}





/* Returns the score rank. */
s8 CheckIfHighscore (u32 score, u32 lines)
{
  short i, j ;
  struct entry *score_table = NULL ;

  switch (global_game.game_type) {
  case Game_Type_Endless:
    score_table = &(global_highscores.endless[0]) ;
    break ;
  case Game_Type_5mn:
    score_table = &(global_highscores.game_5mn[0]) ;
    break ;
  case Game_Type_10mn:
    score_table = &(global_highscores.game_10mn[0]) ;
    break ;
  case Game_Type_15mn:
    score_table = &(global_highscores.game_15mn[0]) ;
    break ;
  }

  if (score_table == NULL) return (-1) ;
  /* Recherche d'un point d'entre dans la table des highscores. */
  for (i = 0; i < NB_HIGHSCORES; i++) {
    if (score > score_table[i].score) {
      /* Found ! Shift lower scores. */
      for (j = (NB_HIGHSCORES - 1); j > i; j--) {
	score_table[j].score = score_table[j - 1].score ;
	score_table[j].lines = score_table[j - 1].lines ;
	strcpy (score_table[j].player, score_table[j - 1].player) ;
      }
      /* Player's name will be written by side effect in "cup.c". */
      score_table[i].score = score ;
      score_table[i].lines = lines ;
      return (i) ;
    }
  }    /* Fin de for (i = 0; i < NB_HIGHSCORES; i++). */

  return (-1) ;
}




void HibernateGame ()
{
  u8 *dest_mem ;
  u32 tag = MAGICNUMBER ;

  /* Remind the elapsed time. */
  global_game.elapsed_semi_seconds = global_elapsed_semi_seconds ;
  /* On commence par sauver les scores et la config. */
  SaveScores () ;
  /* On remet un magic number disant qu'il y a un jeu hibern. */
  dest_mem = ((u8*) SRAM) + sizeof (struct save_data) ;
  /* The magic number. */
  bytes_memcpy (dest_mem, (u8 const*) &tag, sizeof (u32)) ;
  /* Maintenant, on dumpe l'tat du jeu. */
  dest_mem += sizeof (u32) ;
  /* Meta cube. */
  bytes_memcpy (dest_mem, (u8 const*) global_current_metacube,
		sizeof (struct metacube)) ;
  dest_mem += sizeof (struct metacube) ;
  /* Shape. */
  bytes_memcpy (dest_mem, (u8 const*) &current_shape, sizeof (struct shape)) ;
  dest_mem += sizeof (struct shape) ;
  /* Game status */
  bytes_memcpy (dest_mem, (u8 const*) &global_game,
		sizeof (struct game_status)) ;
}





bool LoadHibernated ()
{
  u8 *src_mem ;
  u32 tag ;

  /* It is now safe to read GBA Cartridge memory. */
  WAIT_CR &= ~0x0880 ;

  /* On recherche le premier magic number. */
  src_mem = (u8*) SRAM ;
  bytes_memcpy ((u8*) &tag, src_mem, sizeof (u32)) ;
  if (tag != MAGICNUMBER) return (false) ;

  /* On recherche le second magic number. */
  src_mem = ((u8*) SRAM) + sizeof (struct save_data) ;
  bytes_memcpy ((u8*) &tag, src_mem, sizeof (u32)) ;
  if (tag != MAGICNUMBER) return (false) ;

  /* On dtruit l'hibernation pour la prochaine fois. */
  tag = 0L ;
  bytes_memcpy (src_mem, (u8 const*) &tag, sizeof (u32)) ;
  src_mem += sizeof (u32) ;

  /* Meta cube. */
  bytes_memcpy ((u8*) global_current_metacube, src_mem,
		sizeof (struct metacube)) ;
  src_mem += sizeof (struct metacube) ;
  /* Shape. */
  bytes_memcpy ((u8*) &current_shape, src_mem, sizeof (struct shape)) ;
  src_mem += sizeof (struct shape) ;
  /* Game status. */
  bytes_memcpy ((u8*) &global_game, src_mem, sizeof (struct game_status)) ;

  /* Restore the elapsed time. */
  global_elapsed_semi_seconds = global_game.elapsed_semi_seconds ;
  return (true) ;
}
