#include <nds.h>
#include <string.h>
#include <stdio.h>
#include "info_game_gfx.h"
#include "info_game_fonts_gfx.h"
#include "input_bottom_gfx.h"
#include "char_table.h"



#define BUFFER_LEN (11)
/* Right justified on 10 significant digits. */
static void u32toa (u32 x, char *buffer)
{
  u8 i = 0 ;
  char tmp_c ;
  u32 div = x ;
  u32 remain = 0 ;

  if (x == 0) {
    buffer[0] = '0' ;
    i = 1 ;
  }
  else {
    while (div != 0) {
      remain = div % 10 ;
      div = div / 10 ;
      buffer [i] = '0' + remain ;
      i++ ;
    }
  }

  /* Fill with spaces. */
  for (; i < BUFFER_LEN - 1; i++) buffer[i] = ' ' ;
  /* End with '\0'. */
  buffer[BUFFER_LEN - 1] = '\0' ;

  /* Invert the string except the final '\0'. */
  for (i = 0; i < (BUFFER_LEN - 1) / 2 ; i++) {
    tmp_c = buffer[i] ;
    buffer[i] = buffer[(BUFFER_LEN - 2) - i] ;
    buffer[BUFFER_LEN - 2 - i] = tmp_c ;
  }
}




void LargePrintInt (u16 *dest_map, u32 value, u8 x_tile_offset,
		    u8 y_tile_offset, u16 empty_font_tile_index,
		    u16 zero_font_tile_index, u16 font_size_in_tiles)
{
  u16 tile_on_screen ;
  char buffer[BUFFER_LEN] ;
  u8 i ;
  u16 tile_index ;

  u32toa (value, buffer) ;
  for (i = 0; buffer[i] != '\0'; i++) {
    if (buffer[i] == ' ') tile_index = empty_font_tile_index ;
    else
      tile_index =
	zero_font_tile_index + ((buffer[i] - '0') * font_size_in_tiles) ;
    /* Premire ligne. */
    tile_on_screen =
      (y_tile_offset * (256 / 8)) /* Offset de ligne. */
      /* 256 : Largeur map en pixels. */
      /* 8 : Nbre pixels par tile. */
      + x_tile_offset + i ;  /* Offset de colonne. */
    dest_map[tile_on_screen] = tile_index ;
    /* Seconde ligne. */
    tile_on_screen =
      ((y_tile_offset + 1) * (256 / 8)) /* Offset de ligne. */
      /* 256 : Largeur map en pixels. */
      /* 8 : Nbre pixels par tile. */
      + x_tile_offset + i ;  /* Offset de colonne. */
    dest_map[tile_on_screen] = tile_index + 1 ;
  }
}



void Print2DigitsInt (u16 *dest_map, u8 value, u16 col, u16 line)
{
  s16 column ;
  u8 div ;
  u8 remain = 0 ;
  u16 tile_on_screen, tile_index ;

  div = value ;
  /* Affichage sur 2 chiffres. */
  for (column = 1; column >= 0; column --) {
    remain = div % 10 ;
    div = div / 10 ;
    tile_index =
      (u16) (ZERO_INFO_GAME_FONT_TILE_INDEX +
	     (remain * INFO_GAME_FONT_SIZE_IN_TILES)) ;
    tile_on_screen = (line * (256 / 8)) + (col + column) ;
    /* Premire tile du chiffre. */
    dest_map[tile_on_screen] = tile_index ;
    /* Seconde tile du chiffre. */
    tile_on_screen = ((line + 1) * (256 / 8)) + (col + column) ;
    dest_map[tile_on_screen] = tile_index + 1 ;
  }
}





void SmallPrint (const char *str, u16 *dest_map, u16 font_first_tile_index,
		 u8 at_x, u8 at_y)
{
  short i ;
  char c ;

  for (i = 0; str[i] != '\0'; i++) {
    /* Correction pour tomber au bon indice dans la table des tiles. */
    c = char_to_tile_table[((u16) str[i])] ;
    dest_map [(at_y * (256 / 8)) + at_x] =
      ((u16) c) + font_first_tile_index ;
    at_x++ ;
  }     /* End of for (i = 0; str[i] != '\0'; i++). */
}
